#ifndef _POWERRPC_H
#define _POWERRPC_H
#include <rpc/pwrpcapi.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef void * (*local_call_t) (void*, ...);
typedef void (*free_call_t) (void*);
typedef void (*before_fork_call_t) (void);
typedef void (*pwrpc_perr_call_t) (char*);



enum {POWERRPC_FORKING=1, POWERRPC_THREADING=2, PWRPC_ONC=5, PWRPC_ONC_MT=6};

typedef struct {
    caddr_t         pover;
    int             over_size;
    caddr_t         pback;
    int             back_size;
    SVCXPRT        *ptransport;
	struct          svc_req *psvc_req;
	void            (*proc)(SVCXPRT*);
    local_call_t    local;
    free_call_t     free_after;
    xdrproc_t       xdr_over;
    xdrproc_t       xdr_back;
    int             reply_now;
    int             mt_on_call;
}               pwrpc_call_struct;

PWRPCAPI
void pwrpc_report_event(char*);

PWRPCAPI
void pw_serv_mainloop(SVCXPRT*tcp_master, SVCXPRT* udp_master, int dofork, int exit_idle);

PWRPCAPI
void pw_serv_once(struct timeval *timeout);

PWRPCAPI
void pw_serv_fork(fd_set*, SVCXPRT*, SVCXPRT*);

PWRPCAPI
void pw_fork_on_call(int*);

PWRPCAPI
void pw_serv_init();

PWRPCAPI
void pw_serv_validate();

PWRPCAPI
CLIENT * pw_clnt_create(const char*host, short port, u_long prog, u_long vers, char* proto, u_int sendsz, u_int recvsz);

PWRPCAPI
SOCKET pw_serv_fd(u_short port,const char* type);



#if 0

PWRPCAPI
void pw_serv_input_handler(int fd, void (*handler)(int fd));

PWRPCAPI
void pw_serv_output_handler(int fd, void (*handler)(int fd));

PWRPCAPI
void pw_serv_exception_handler(int fd, void (*handler)(int fd));

#endif

PWRPCAPI
void       pw_serv_call(pwrpc_call_struct * pcall);

PWRPCAPI
void pw_set_err_report_func(pwrpc_perr_call_t err_call);

PWRPCAPI
void pw_free_reference(xdrproc_t, void*, u_int size);

#ifndef _WIN32_WCE
PWRPCAPI
int pw_serialize(FILE* fp, xdrproc_t xdrproc, void * pdata, enum xdr_op op);
#endif

PWRPCAPI
enum clnt_stat pw_clnt_errno(CLIENT *cl);

PWRPCAPI bool_t	xdr_array_pw(XDR*, caddr_t*, u_int*, u_int, u_int, xdrproc_t);
PWRPCAPI bool_t	xdr_bytes_pw(XDR*, char**, u_int*, u_int);

#ifdef __cplusplus
};
#endif

#endif
