#include <rpc/rpc.h>
#include <rpc/pmap_cln.h>
#define _RPC_SERVER
#include "msg.h"
#include <powerrpc.h>
static SVCXPRT *tcp_master =0;
static SVCXPRT *udp_master =0;
#undef _RPCAPI
#define _RPCAPI MSGSERVAPI
#define MSGSERVAPI __declspec(dllexport)
#define RPC_MT_CONNECT 1
#define RPC_RETURN_ARG 1
/******************************************
The two macros below show how to use fixed port
for server, and how to bypass portmapper
 registration
*******************************************/
#define MSGSERV_V1_tcp_server_port 2244

static void
msgserv_1(struct svc_req* rqstp, SVCXPRT *transp)
{
	union {
		char *sendmsg_1_OVER;
		intarr sendintegers_1_OVER;
		double send_double_1_OVER;
	} RPC_over;
	union {
		char *sendmsg_1_BACK;
		intarr sendintegers_1_BACK;
		int send_double_1_BACK;
	} RPC_back;
	pwrpc_call_struct callInfo;
	callInfo.ptransport = transp;
	callInfo.pover = (caddr_t)&RPC_over;
	callInfo.pback = (caddr_t)&RPC_back;
	callInfo.reply_now = 0;
#ifndef PWRPC_MT_ON_CALL
#define PWRPC_MT_ON_CALL 5
#endif
	callInfo.mt_on_call = PWRPC_MT_ON_CALL;
	callInfo.free_after = 0;
	callInfo.proc = 0;
	callInfo.psvc_req = rqstp;

	switch (rqstp->rq_proc) {
	case NULLPROC:
		(void)svc_sendreply(transp, (xdrproc_t)xdr_void, (char *)NULL);
		return;

	case sendmsg:
		callInfo.xdr_over = (xdrproc_t) xdr_wrapstring;
		callInfo.over_size = sizeof(RPC_over);
		callInfo.xdr_back = (xdrproc_t)xdr_wrapstring;
		callInfo.back_size = sizeof(RPC_back);
		callInfo.local = (local_call_t) sendmsg_1_svc;
		break;

	case sendintegers:
		callInfo.xdr_over = (xdrproc_t) xdr_intarr;
		callInfo.over_size = sizeof(RPC_over);
		callInfo.xdr_back = (xdrproc_t)xdr_intarr;
		callInfo.back_size = sizeof(RPC_back);
		callInfo.local = (local_call_t) sendintegers_1_svc;
		break;

	case send_double:
		callInfo.xdr_over = (xdrproc_t) xdr_double;
		callInfo.over_size = sizeof(RPC_over);
		callInfo.xdr_back = (xdrproc_t)xdr_int;
		callInfo.back_size = sizeof(RPC_back);
		callInfo.local = (local_call_t) send_double_1_svc;
		break;

	default:
		svcerr_noproc(transp);
		return;
	}
	pw_serv_call(&callInfo);

}



int main(int argc, char**argv)
{
	SVCXPRT *transp;
	int sock = RPC_ANYSOCK;
	int proto = 0;
	u_long _program_num, _version_num;
	short tcp_port=0, udp_port =0;
	pw_serv_init();
	(void)pmap_unset(MSGSERV, MSGSERV_V1);

	sock=RPC_ANYSOCK;

#ifndef RPC_MT_CONNECT
#define RPC_MT_CONNECT 0
#endif
	pw_set_mt_on_connect(RPC_MT_CONNECT);

/*Define MSGSERV_V1_udp_server_port to used fixed port for this service*/
#ifdef MSGSERV_V1_udp_server_port
	udp_port = MSGSERV_V1_udp_server_port;
#endif

	if(udp_port)
		sock = pw_serv_fd(udp_port, "udp");
	transp = svcudp_create(sock);
	if (transp == NULL) {
		server_err("cannot create udp service.\n");
		server_exit(1);
	}
		server_msg("Created udp service.\n");
	udp_master = transp;
	_program_num = MSGSERV;
	_version_num = MSGSERV_V1;

/*Give a chance to perform initializations*/

#ifdef MSGSERV_init
	if(0 > MSGSERV_init(argc, argv, &_program_num, &_version_num, transp, "udp"));

#endif

/*Define MSGSERV_V1_udp_no_pmap to skip portmapper registration*/

	proto=0;

#ifndef MSGSERV_V1_udp_no_pmap
	proto = IPPROTO_UDP;
#endif

	if (!svc_register(transp, _program_num, _version_num, msgserv_1, proto)) {
		server_err("unable to register (MSGSERV, MSGSERV_V1, udp).\n");
		server_exit(1);
	}

	sock=RPC_ANYSOCK;

#ifndef RPC_MT_CONNECT
#define RPC_MT_CONNECT 0
#endif
	pw_set_mt_on_connect(RPC_MT_CONNECT);

/*Define MSGSERV_V1_tcp_server_port to used fixed port for this service*/
#ifdef MSGSERV_V1_tcp_server_port
	tcp_port = MSGSERV_V1_tcp_server_port;
#endif

	if(tcp_port)
		sock = pw_serv_fd(tcp_port, "tcp");
	transp = svctcp_create(sock, 0, 0);
	if (transp == NULL) {
		server_err("cannot create tcp service.\n");
		server_exit(1);
	}
		server_msg("Created tcp service.\n");
	tcp_master = transp;
	_program_num = MSGSERV;
	_version_num = MSGSERV_V1;

/*Give a chance to perform initializations*/

#ifdef MSGSERV_init
	if(0 > MSGSERV_init(argc, argv, &_program_num, &_version_num, transp, "tcp"));

#endif

/*Define MSGSERV_V1_tcp_no_pmap to skip portmapper registration*/

	proto=0;

#ifndef MSGSERV_V1_tcp_no_pmap
	proto = IPPROTO_TCP;
#endif

	if (!svc_register(transp, _program_num, _version_num, msgserv_1, proto)) {
		server_err("unable to register (MSGSERV, MSGSERV_V1, tcp).\n");
		server_exit(1);
	}
	pw_serv_mainloop(tcp_master, udp_master, 0, 1);
	server_err("svc_run returned\n");
	exit(1);return 0;
}
